
ViewManager subclass: #PhoneBookApp
  instanceVariableNames: 
    'phones selectedName listPane textPane '
  classVariableNames: ''
  poolDictionaries: 
    'ColorConstants '   !


!PhoneBookApp class methods !

example
    self new openOn: Dictionary new! !



!PhoneBookApp methods !
  
add
    "Add a new name to the dictionary."
    | key |
    key := Prompter prompt: 'New name' default: ''.
    (key isNil)  | (key isEmpty)
        ifTrue: [^Menu message: 'Name is blank -- addition cancelled'].
    (phones includesKey: key)
        ifTrue: [^Menu message: key, ' already in phone book'].
    selectedName := key.
    phones at: key put: String new.
    self
        list: listPane;
        text: textPane!

debug
    self halt!
   
list: aListPane
    "Fill the list pane with the names."
    aListPane
        contents: phones keys asSortedCollection;
        selection: selectedName!
   
nameSelected: aListPane
    "Display the phone number for the selected name in the list pane."
    selectedName := aListPane selectedItem.
    self text: textPane!
  
openOn: aDictionary
    selectedName := nil.
    phones := aDictionary.
    self label: 'Phone Book'.
    self addSubpane:
        (listPane := ListPane new
            owner: self;
            backColor: ClrLightgray;
            foreColor: ClrBlue;
            when: #getContents perform: #list:;
            when: #select perform: #nameSelected:;
            framingRatio: (0.1 @ 0.3 extent: 0.3 @ 0.4)).
     self addSubpane:
        (textPane := TextPane new
            owner: self;
            backColor: ClrLightgray;
            foreColor: ClrDarkred;
            when: #getContents perform: #text:;
            when: #save perform: #textFrom:;
            framingRatio: (0.5 @ 0.1 extent: 0.4 @ 0.8)).
    self mainView menuWindow addMenu: (
        Menu new
            title: '~Phones';
            owner: self;
            appendItem: '~Add' selector: #add;
            appendItem: '~Remove' selector: #remove).
    self mainView menuWindow addMenu: (
        Menu new
            title: 'Debugger';
            owner: self;
            appendItem: 'Debug' selector: #debug).
    self openWindow!

remove
    "Remove the selected name from the dictionary."
    selectedName isNil ifTrue: [^Menu message: 'Select a name first'].
    phones removeKey: selectedName.
    selectedName := nil.
    self
        list: listPane;
        text: textPane!
  
text: aTextPane
    "Set the contents of the text pane to the string
    associated with the selected name."
    aTextPane contents: (phones at: selectedName ifAbsent: [''])!
   
textFrom: aTextPane
    "Get the contents of the text pane as the phone number for
    the selected name and enter into the dictionary."
    selectedName isNil ifTrue: [
        Menu message: 'No name selected -- save ignored'.
        ^self changed: #text:].
    phones at: selectedName put: aTextPane contents.
    aTextPane modified: false! !
