
ViewManager subclass: #AnimationPaneDemo
  instanceVariableNames: 
    'animationPane objAtCursor '
  classVariableNames: ''
  poolDictionaries: 
    'ColorConstants WBConstants '  !


!AnimationPaneDemo class methods !
   
example1
        "To run a demo execute:  AnimationPaneDemo example1 "
    self new open!
 
wbCreated
    ^true! !



!AnimationPaneDemo methods !
 
animate
        "Animate the object at the current mouse location."
    objAtCursor notNil ifTrue: [objAtCursor animate]


    "KEN -- If an object is dragged and then animated, its new
     speed and direction are related to the manner in which it
     was dragged.  Do you know why?  If so, the might mention
     the reason in the book."!
   
animateAll
        "Animate all of the objects in the pane."
    animationPane contents do: [:object | object animate]!
   
continue
        "Exit the menu while doing nothing"!
  
createBouncer
        "Create a bouncer at the current mouse location."
    (self createObjectWithColor: ClrBlue) bouncer!

createChaser
        "Create a chaser at the current mouse location."
    | object |
    object := self createObjectWithColor: ClrGreen.
    object chaser;
               endBlock: [:blockArg | Menu message: 'Gotcha!!']!
   
createObjectWithColor: aColor
        "Create an animated object at the current  mouse location.
         The object has two images consisting of rectangles with different orientations.
         The background color of the rectangles is aColor.
         The method returns the object."

    | bitmap object polyPoints |

    "Create the object."
    object := AnimatedObject new.
    object position: animationPane mouseLocation.
    animationPane addObject: object.

    "Create the first image."
    polyPoints := OrderedCollection new.
    polyPoints add:7@7; add:29@7; add:29@29; add:7@29; add:7@7.
    bitmap := Bitmap new ownDCScreenExtent: 36 @ 36.
    bitmap pen
        foreColor: ClrRed;
        backColor: aColor;
        setLineWidth: 5;
        polygonFilled: polyPoints.
    object addFrame: bitmap.

    "Create the second image."
    polyPoints := OrderedCollection new.
    polyPoints add:2.6@18; add:18@2.6; add:33.4@18; add:18@33.4; add:2.6@18.
    bitmap := Bitmap new ownDCScreenExtent: 36 @ 36.
    bitmap pen
        foreColor: ClrRed;
        backColor: aColor;
        setLineWidth: 5;
        polygonFilled: polyPoints.
    object addFrame: bitmap.

    "Tell the object how many steps to take before changing images,
      and display the object."
    object stepsPerFrame: 20;
               display.

    ^object!
  
erase
        "Erase  the object at the current mouse location, remove
         it from the animation pane, and release the object's frames."
    objAtCursor notNil ifTrue: [
        objAtCursor hide.
        (objAtCursor active) ifTrue: [objAtCursor stop].
        animationPane contents remove: objAtCursor.
        objAtCursor frames do: [:frame | frame release]]!
  
eraseAll
        "Erase  all the objects from the pane."
    animationPane stopAll.
    animationPane contents do: [: object |
            object hide.
            object frames do: [:frame | frame release]].
    animationPane contents: OrderedCollection new!
   
onButton1Down: aPane
        "If there is an object at the cursor, then prepare to drag it."
    objAtCursor := animationPane childAt: animationPane mouseLocation.
    objAtCursor notNil
        ifTrue: [animationPane when: #button1Move perform: #onButton1MoveDragObject:]!
   
onButton1MoveDragObject: aPane
        "Drag the object at the current mouse location."
    objAtCursor drag: animationPane mouseLocation.
    objAtCursor active ifTrue: [objAtCursor stop].
    animationPane when: #button1Move perform: nil!

onButton2Down: aPane
        "Pop up a menu."
    | command |
    objAtCursor := animationPane childAt: animationPane mouseLocation.
    command := self popUpMenu.
    command isNil ifFalse: [self perform: command]!

onClose: aPane
        "Release all frames and close window."
    animationPane contents
        do: [:object | object frames do: [:frame | frame release]].
    self close!

open
        "Open a window on an animation pane demo."
    self
        labelWithoutPrefix: 'Animation Pane Demo';
        when: #close perform: #onClose:.
    self addSubpane:
        (animationPane := AnimationPane new
            owner: self;
            when: #button1Down perform: #onButton1Down:;
            when: #button1Move perform: nil;
            when: #button2Down perform: #onButton2Down:;
            framingRatio: (0.0 @ 0.0 extent: 1 @ 1)).
    self mainView menuWindow addMenu: (
        Menu new
            title: 'All Rectangles';
            owner: self;
            appendItem: 'Animate' selector: #animateAll;
            appendItem: 'Stop' selector: #stopAll;
            appendItem: 'Erase' selector: #eraseAll).
    self openWindow!

popUpMenu
        "Pops up menu of commands."
    | menu |
    menu := Menu
        labels: (
            'Create Bouncer\',
            'Create Chaser\',
            'Animate\',
            'Stop\',
            'Erase\',
            'Continue') withCrs
        lines: #(2 5)
        selectors: #(
            createBouncer
            createChaser
           animate
            stop
            erase
            continue).
    ^menu popUp!
   
stop
        "Stop the animation of the object at the current mouse location."
    objAtCursor notNil ifTrue: [
        (objAtCursor active) ifTrue: [objAtCursor stop]]!

stopAll
        "Stop the animation of all of the objects in the pane."
    animationPane stopAll! !
